/*
 * @author     Ekawat (ice) Homsirikamol
 * @version    1.0
 * @copyright  Copyright (c) 2017 Cryptographic Engineering Research Group
 *             ECE Department, George Mason University Fairfax, VA, U.S.A.
 *             All rights Reserved.
 * @license    This project is released under the GNU Public License.
 *             The license and distribution terms for this file may be
 *             found in the file LICENSE in this distribution or at
 *             http://www.gnu.org/licenses/gpl-3.0.txt
 * @note       This implementation is derived in part from the reference implementation
 *             included in SUPERCOP-20171020, with the list of co-authors listed in ref/designers
 *             
 *             This is publicly available encryption source code that falls
 *             under the License Exception TSU (Technology and software-
 *             unrestricted)
 */

#include "ap_int.h"
#include "hls_interface.h"

#ifndef HLS_UTILS_H
#define HLS_UTILS_H

#if !__SYNTHESIS__
void pp(const char *txt, array_t data);
#endif

void swap_endian(array_t c, array_t a, const int word_bytes);
void byte_valid(array_t c, array_t a, data_bytes_t vld);
void byte_valid_pad(array_t c, array_t a, data_bytes_t vld, data_bytes_t pad);
void clear(array_t a);
void copy(array_t b, array_t a);
void display(array_t a);
bool match(array_t b, array_t a);
data_t reshape_array(array_t word);
void reshape_block(array_t word, data_t block);
void rot_left(array_t out, array_t in, ap_uint<4> size);
void shf_left(array_t out, array_t in, ap_uint<4> size);
void rot_right(array_t out, array_t in, ap_uint<4> size);
void shf_right(array_t out, array_t in, ap_uint<4> size);
void shf_left1(array_t out, array_t in);
void shf_right1(array_t out, array_t in);
void barrel_shf_right1(array_t out, array_t in, ap_uint<7> size);

void xor2(array_t c, array_t a, array_t b);
void xor3(array_t d, array_t a, array_t b, array_t c);

#endif /* HLS_UTILS_H */
